@@ -48,11 +48,11 @@ struct Restraints {
         if (altloc2 != '\0')
           alt = altloc2;
       }
-      Atom* a = residue->find_atom(atom, alt);
+      Atom* a = residue->find_atom(atom, alt, El::X, false);
       // Special case: microheterogeneity may have shared atoms only in
       // the first residue. Example: in 1ejg N is shared between PRO and SER.
       if (a == nullptr && alt != '\0' && residue->group_idx > 0)
-        a = (residue - residue->group_idx)->find_atom(atom, alt);
+        a = (residue - residue->group_idx)->find_atom(atom, alt, El::X, false);
       return a;
     }
     const Atom* get_from(const Residue& res1, const Residue* res2,
@@ -63,12 +63,13 @@ struct Restraints {
 
   static std::string lexicographic_str(const std::string& name1,
                                        const std::string& name2) {
-    return name1 < name2 ? name1 + "-" + name2 : name2 + "-" + name1;
+    return name1 < name2 ? cat(name1, '-', name2) : cat(name2, '-', name1);
   }
 
   enum class DistanceOf { ElectronCloud, Nucleus };
 
   struct Bond {
+    static const char* what() { return "bond"; }
     AtomId id1, id2;
     BondType type;
     bool aromatic;
@@ -76,7 +77,7 @@ struct Restraints {
     double esd;
     double value_nucleus;
     double esd_nucleus;
-    std::string str() const { return id1.atom + "-" + id2.atom; }
+    std::string str() const { return cat(id1.atom, '-', id2.atom); }
     std::string lexicographic_str() const {
       return Restraints::lexicographic_str(id1.atom, id2.atom);
     }
@@ -91,27 +92,30 @@ struct Restraints {
   };
 
   struct Angle {
+    static const char* what() { return "angle"; }
     AtomId id1, id2, id3;
     double value;  // degrees
     double esd;
     double radians() const { return rad(value); }
     std::string str() const {
-      return id1.atom + "-" + id2.atom + "-" + id3.atom;
+      return cat(id1.atom, '-', id2.atom, '-', id3.atom);
     }
   };
 
   struct Torsion {
+    static const char* what() { return "torsion"; }
     std::string label;
     AtomId id1, id2, id3, id4;
     double value;
     double esd;
     int period;
     std::string str() const {
-      return id1.atom + "-" + id2.atom + "-" + id3.atom + "-" + id4.atom;
+      return cat(id1.atom, '-', id2.atom, '-', id3.atom, '-', id4.atom);
     }
   };
 
   struct Chirality {
+    static const char* what() { return "chirality"; }
     AtomId id_ctr, id1, id2, id3;
     ChiralityType sign;
 
@@ -120,11 +124,12 @@ struct Restraints {
              (sign == ChiralityType::Negative && volume > 0);
     }
     std::string str() const {
-      return id_ctr.atom + "," + id1.atom + "," + id2.atom + "," + id3.atom;
+      return cat(id_ctr.atom, ',', id1.atom, ',', id2.atom, ',', id3.atom);
     }
   };
 
   struct Plane {
+    static const char* what() { return "plane"; }
     std::string label;
     std::vector<AtomId> ids;
     double esd;
@@ -306,7 +311,7 @@ inline double chiral_abs_volume(double bond1, double bond2, double bond3,
     x -= cosine * cosine;
     y *= cosine;
   }
-  return mult * std::sqrt(x + y);
+  return mult * std::sqrt(std::max(0., x + y));
 }
 
 inline double Restraints::chiral_abs_volume(const Restraints::Chirality& ch) const {
@@ -336,6 +341,7 @@ struct ChemComp {
 
   struct Atom {
     std::string id;
+    std::string old_id;  // read from _chem_comp_atom.alt_atom_id
     Element el;
     // _chem_comp_atom.partial_charge can be non-integer,
     // _chem_comp_atom.charge is always integer (but sometimes has format
@@ -430,6 +436,18 @@ struct ChemComp {
     return find_atom(atom_id) != atoms.end();
   }
 
+  std::vector<Atom>::iterator find_atom_by_old_name(const std::string& old_id) {
+    return std::find_if(atoms.begin(), atoms.end(),
+                        [&](const Atom& a) { return a.old_id == old_id; });
+  }
+  std::vector<Atom>::const_iterator find_atom_by_old_name(const std::string& old_id) const {
+    return const_cast<ChemComp*>(this)->find_atom_by_old_name(old_id);
+  }
+  bool has_old_names() const {
+    return std::any_of(atoms.begin(), atoms.end(),
+                       [&](const Atom& a) { return !a.old_id.empty() && a.old_id != a.id; });
+  }
+
   int get_atom_index(const std::string& atom_id) const {
     auto it = find_atom(atom_id);
     if (it == atoms.end())
@@ -575,8 +593,10 @@ inline ChemComp make_chemcomp_from_block(const cif::Block& block_) {
     cc.type_or_group = type_col.str(0);
   for (auto row : block.find("_chem_comp_atom.",
                              {"atom_id", "type_symbol", "?type_energy",
-                             "?charge", "?partial_charge"}))
-    cc.atoms.push_back({row.str(0), Element(row.str(1)),
+                             "?charge", "?partial_charge", "?alt_atom_id"}))
+    cc.atoms.push_back({row.str(0),
+                        row.has(5) ? row.str(5) : "",
+                        Element(row.str(1)),
                         (float) cif::as_number(row.one_of(3, 4), 0.0),
                         row.has(2) ? row.str(2) : "",
                         Position()});
