@@ -34,7 +34,7 @@ inline bool atom_match_with_alias(const std::string& atom_id, const std::string&
   return atom_id == atom;
 }
 
-struct ChemLink {
+struct GEMMI_DLL ChemLink {
   struct Side {
     using Group = ChemComp::Group;
     std::string comp;
@@ -266,7 +266,7 @@ struct GEMMI_DLL MonLib {
 
   void set_monomer_dir(const std::string& monomer_dir_) {
     monomer_dir = monomer_dir_;
-    if (monomer_dir.back() != '/' && monomer_dir.back() != '\\')
+    if (!monomer_dir.empty() && monomer_dir.back() != '/' && monomer_dir.back() != '\\')
       monomer_dir += '/';
   }
 
