@@ -7,6 +7,7 @@
 #include "gemmi/align.hpp"     // for assign_label_seq_id
 #include "gemmi/mmread_gz.hpp" // for read_structure_gz
 #include "gemmi/enumstr.hpp"   // for polymer_type_to_string, entity_type_to_string
+#include "gemmi/resinfo.hpp"   // for find_tabulated_residue
 #include "histogram.h"         // for terminal_columns
 
 #define GEMMI_PROG residues
@@ -151,6 +152,8 @@ void print_long_info(const gemmi::Model& model, OptParser& p) {
           prev = &at.name;
         } else {
           if (print_alt) {
+            if GEMMI_UNLIKELY((&at-1)->altloc == '\0')
+              putchar(':');
             putchar(',');
             if (at.altloc)
               putchar(at.altloc);
