@@ -21,7 +21,7 @@ using gemmi::Topo;
 
 namespace {
 
-enum OptionIndex { Quiet=AfterMonLibOptions, FormatIn, Cutoff, Sort };
+enum OptionIndex { Quiet=AfterMonLibOptions, FormatIn, Cutoff, Sort, Missing };
 
 const option::Descriptor Usage[] = {
   { NoOp, 0, "", "", Arg::None,
@@ -42,6 +42,8 @@ const option::Descriptor Usage[] = {
     "  --cutoff=ZC  \tList bonds and angles with Z score > ZC (default: 2)." },
   { Sort, 0, "s", "sort", Arg::None,
     "  -s, --sort  \tSort output according to |Z|." },
+  { Missing, 0, "", "missing", Arg::None,
+    "  --missing  \tList missing atoms." },
   MonLibUsage[2], // details about Libin (--lib)
   { 0, 0, 0, 0, 0, 0 }
 };
@@ -72,6 +74,7 @@ struct RMSes {
 
 void check_restraint(const Topo::Rule rule,
                      const Topo& topo,
+                     const gemmi::UnitCell& cell,
                      double cutoff,
                      const char* tag,
                      RMSes* rmses,
@@ -94,7 +97,10 @@ void check_restraint(const Topo::Rule rule,
   switch (rule.rkind) {
     case Topo::RKind::Bond: {
       const Topo::Bond& t = topo.bonds[rule.index];
-      z = t.calculate_z();
+      double dist = t.calculate();
+      if (t.asu == gemmi::Asu::Different)
+        dist = cell.find_nearest_image(t.atoms[0]->pos, t.atoms[1]->pos, t.asu).dist();
+      z = t.calculate_z_(dist);
       if (z > cutoff) {
         rmses->wrong_bond++;
         if (verbosity >= 0) {
@@ -221,9 +227,7 @@ int GEMMI_MAIN(int argc, char **argv) {
         fprintf(stderr, "No atoms in the input file. Wrong format?\n");
         return 1;
       }
-      if (st.input_format == gemmi::CoorFormat::Pdb ||
-          st.input_format == gemmi::CoorFormat::ChemComp)
-        gemmi::setup_entities(st);
+      gemmi::setup_entities(st);
 
       gemmi::MonLib monlib;
       std::vector<std::string> wanted = st.models[0].get_all_residue_names();
@@ -239,6 +243,14 @@ int GEMMI_MAIN(int argc, char **argv) {
         topo.initialize_refmac_topology(st, model, monlib);
         topo.apply_all_restraints(monlib);
 
+        if (p.options[Missing]) {
+          std::vector<gemmi::AtomAddress> vec = find_missing_atoms(topo);
+          printf("%zu missing atoms.\n", vec.size());
+          for (const gemmi::AtomAddress& aa : vec)
+            printf("    %s\n", aa.str().c_str());
+          continue;
+        }
+
         RMSes rmses;
         std::multimap<double, std::string> line_storage;
         std::multimap<double, std::string>* lines = nullptr;
@@ -254,18 +266,21 @@ int GEMMI_MAIN(int argc, char **argv) {
               std::string rtag = gemmi::cat(chain_info.chain_ref.name, ' ',
                                             prev.res1->str(), '-', ri.res->str());
               for (const Topo::Rule& rule : prev.link_rules)
-                check_restraint(rule, topo, cutoff, rtag.c_str(), &rmses, verbosity, lines);
+                check_restraint(rule, topo, st.cell, cutoff, rtag.c_str(),
+                                &rmses, verbosity, lines);
             }
             std::string rtag = chain_info.chain_ref.name + " ";
             if (st.input_format != gemmi::CoorFormat::ChemComp)
               rtag += ri.res->seqid.str();
             gemmi::cat_to(rtag, '(', ri.res->name,  ')');
             for (const Topo::Rule& rule : ri.monomer_rules)
-              check_restraint(rule, topo, cutoff, rtag.c_str(), &rmses, verbosity, lines);
+              check_restraint(rule, topo, st.cell, cutoff, rtag.c_str(),
+                              &rmses, verbosity, lines);
           }
         for (const Topo::Link& link : topo.extras)
           for (const Topo::Rule& rule : link.link_rules)
-            check_restraint(rule, topo, cutoff, "link", &rmses, verbosity, lines);
+            check_restraint(rule, topo, st.cell, cutoff, "link",
+                            &rmses, verbosity, lines);
 
         if (lines) {
           for (const auto& t : *lines)
