@@ -1,7 +1,7 @@
 // Copyright 2017 Global Phasing Ltd.
 
+#include <sstream>
 #include "gemmi/cifdoc.hpp"
-#include "tostr.hpp"
 #include "gemmi/to_cif.hpp"
 #include "gemmi/to_json.hpp"
 #include "gemmi/fstream.hpp"
@@ -70,7 +70,21 @@ void add_cif(py::module& cif) {
     .value("PreferPairs", Style::PreferPairs)
     .value("Pdbx", Style::Pdbx)
     .value("Indent35", Style::Indent35)
-    .value("Aligned", Style::Aligned);
+    .value("Aligned", Style::Aligned)
+    ;
+  py::class_<WriteOptions>(cif, "WriteOptions")
+    .def(py::init<>())
+    .def_readwrite("prefer_pairs", &WriteOptions::prefer_pairs)
+    .def_readwrite("compact", &WriteOptions::compact)
+    .def_readwrite("misuse_hash", &WriteOptions::misuse_hash)
+    .def_readwrite("align_pairs", &WriteOptions::align_pairs)
+    .def_readwrite("align_loops", &WriteOptions::align_loops)
+    .def("__repr__", [](const WriteOptions &self) -> std::string {
+        std::string str = self.str();
+        if (str.empty())
+          return "gemmi.cif.WriteOptions()";
+        return gemmi::cat("<gemmi.cif.WriteOptions ", str, '>');
+    });
   py::class_<Document>(cif, "Document")
     .def(py::init<>())
     .def_readwrite("source", &Document::source)
@@ -121,16 +135,27 @@ void add_cif(py::module& cif) {
          py::arg("name"),
          py::return_value_policy::reference_internal)
     .def("write_file",
-         [](const Document& doc, const std::string& filename, Style s) {
+         [](const Document& doc, const std::string& filename, WriteOptions opt) {
+        gemmi::Ofstream os(filename);
+        write_cif_to_stream(os.ref(), doc, opt);
+    }, py::arg("filename"), py::arg("options")=WriteOptions(),
+    "Write data to a CIF file.")
+    // deprecated
+    .def("write_file", [](const Document& doc, const std::string& filename, Style s) {
         gemmi::Ofstream os(filename);
         write_cif_to_stream(os.ref(), doc, s);
-    }, py::arg("filename"), py::arg("style")=Style::Simple,
-    "Write data to a CIF file.")
-    .def("as_string", [](const Document& d, Style style) {
+    }, py::arg("filename"), py::arg("style"))
+    .def("as_string", [](const Document& d, WriteOptions opt) {
         std::ostringstream os;
-        write_cif_to_stream(os, d, style);
+        write_cif_to_stream(os, d, opt);
         return os.str();
-    }, py::arg("style")=Style::Simple, "Returns a string in CIF format.")
+    }, py::arg("options")=WriteOptions(), "Returns a string in CIF format.")
+    // deprecated
+    .def("as_string", [](const Document& d, Style s) {
+        std::ostringstream os;
+        write_cif_to_stream(os, d, s);
+        return os.str();
+    }, py::arg("style"))
     .def("as_json", [](const Document& d, bool mmjson, bool lowercase_names) {
         std::ostringstream os;
         JsonWriter writer(os);
@@ -266,18 +291,29 @@ void add_cif(py::module& cif) {
            return data;
          }, py::arg("name"), py::arg("raw")=false)
     .def("write_file",
-         [](const Block& self, const std::string& filename, Style s) {
+         [](const Block& self, const std::string& filename, WriteOptions opt) {
+        gemmi::Ofstream os(filename);
+        write_cif_block_to_stream(os.ref(), self, opt);
+    }, py::arg("filename"), py::arg("options")=WriteOptions(),
+    "Write data to a CIF file.")
+    // deprecated
+    .def("write_file", [](const Block& self, const std::string& filename, Style s) {
         gemmi::Ofstream os(filename);
         write_cif_block_to_stream(os.ref(), self, s);
-    }, py::arg("filename"), py::arg("style")=Style::Simple,
-    "Write data to a CIF file.")
-    .def("as_string", [](const Block& self, Style style) {
+    }, py::arg("filename"), py::arg("style"))
+    .def("as_string", [](const Block& self, WriteOptions opt) {
         std::ostringstream os;
-        write_cif_block_to_stream(os, self, style);
+        write_cif_block_to_stream(os, self, opt);
         return os.str();
-    }, py::arg("style")=Style::Simple, "Returns a string in CIF format.")
+    }, py::arg("options")=WriteOptions(), "Returns a string in CIF format.")
+    // deprecated
+    .def("as_string", [](const Block& self, Style s) {
+        std::ostringstream os;
+        write_cif_block_to_stream(os, self, s);
+        return os.str();
+    }, py::arg("style"))
     .def("__repr__", [](const Block &self) {
-        return gemmi::tostr("<gemmi.cif.Block ", self.name, '>');
+        return gemmi::cat("<gemmi.cif.Block ", self.name, '>');
     });
 
 
@@ -303,13 +339,22 @@ void add_cif(py::module& cif) {
     .def("length", &Loop::length, "Returns number of rows")
     .def_readonly("tags", &Loop::tags)
     .def_readonly("values", &Loop::values)
-    .def("val", &Loop::val, py::arg("row"), py::arg("col"))
+    .def("__getitem__", [](Loop &self, std::pair<int,int> idx) {
+        return self.val(c_index(idx.first, self.length()),
+                        c_index(idx.second, self.width()));
+    })
+    .def("__setitem__", [](Loop &self, std::pair<int,int> idx, std::string value) {
+        self.val(c_index(idx.first, self.length()),
+                 c_index(idx.second, self.width())) = value;
+    })
     .def("add_row", &Loop::add_row<std::vector<std::string>>,
          py::arg("new_values"), py::arg("pos")=-1)
+    .def("add_columns", &Loop::add_columns,
+         py::arg("column_names"), py::arg("value"), py::arg("pos")=-1)
+    .def("remove_column", &Loop::remove_column)
     .def("set_all_values", &Loop::set_all_values, py::arg("columns"))
     .def("__repr__", [](const Loop &self) {
-        return gemmi::tostr("<gemmi.cif.Loop ", self.length(), " x ",
-                            self.width(), '>');
+        return gemmi::cat("<gemmi.cif.Loop ", self.length(), " x ", self.width(), '>');
     });
 
 
@@ -332,11 +377,14 @@ void add_cif(py::module& cif) {
         self.at(idx) = value;
     })
     .def("str", &Column::str, py::arg("index"))
+    .def("erase", &Column::erase)
     .def("__repr__", [](const Column &self) {
-        std::string desc = "nil";
+        std::string s = "<gemmi.cif.Column ";
         if (const std::string* tag = self.get_tag())
-          desc = gemmi::tostr(*tag, " length ", self.length());
-        return "<gemmi.cif.Column " + desc + ">";
+          gemmi::cat_to(s, *tag, " length ", self.length(), '>');
+        else
+          s += "nil>";
+        return s;
     });
 
   cif_table
@@ -366,10 +414,12 @@ void add_cif(py::module& cif) {
     .def("__bool__", &Table::ok)
     .def("__len__", &Table::length)
     .def("__repr__", [](const Table& self) {
-        return "<gemmi.cif.Table " +
-               (self.ok() ? gemmi::tostr(self.length(), " x ", self.width())
-                          : "nil") +
-               ">";
+        std::string s = "<gemmi.cif.Table ";
+        if (self.ok())
+          gemmi::cat_to(s, self.length(), " x ", self.width(), '>');
+        else
+          s += "nil>";
+        return s;
     });
 
   cif_table_row
