@@ -8,7 +8,7 @@
 #include "gemmi/reciproc.hpp" // for count_reflections, make_miller_vector
 #include "gemmi/cif2mtz.hpp"  // for CifToMtz
 #include "gemmi/mtz2cif.hpp"  // for MtzToCif
-#include "gemmi/merge.hpp"    // for Intensities
+#include "gemmi/intensit.hpp" // for Intensities
 #include "gemmi/binner.hpp"   // for Binner
 #include "gemmi/ecalc.hpp"    // for calculate_amplitude_normalizers
 
@@ -56,8 +56,6 @@ void add_hkl(py::module& m) {
          [](ReflnBlock& self, const std::string& tag, double null) {
            return py_array_from_vector(self.make_vector(tag, null));
     }, py::arg("tag"), py::arg("null")=NAN)
-    .def("make_float_array", &ReflnBlock::make_vector<double>,
-         py::arg("tag"), py::arg("null")=NAN)
     .def("make_miller_array", [](ReflnBlock& self) {
         return py::array_t<int>(py::cast(self.make_miller_vector()));
     })
@@ -204,6 +202,8 @@ void add_hkl(py::module& m) {
     .def("remove_systematic_absences", &Intensities::remove_systematic_absences)
     .def("merge_in_place", &Intensities::merge_in_place, py::arg("itype"))
     .def("read_mtz", &Intensities::read_mtz, py::arg("mtz"), py::arg("type"))
+    .def("prepare_merged_mtz", &Intensities::prepare_merged_mtz,
+         py::arg("with_nobs"))
     .def_property_readonly("miller_array", [](const Intensities& self) {
       const Intensities::Refl* data = self.data.data();
       py::array::ShapeContainer shape({(py::ssize_t)self.data.size(), 3});
