@@ -3,17 +3,15 @@
 #include "gemmi/numb.hpp"
 #include "gemmi/cifdoc.hpp"
 #include "gemmi/cif.hpp"
-#include "gemmi/json.hpp"
+#include "gemmi/mmcif.hpp"         // for make_structure_from_block
+#include "gemmi/pdb.hpp"           // for read_pdb_string
+#include "gemmi/gz.hpp"            // for estimate_uncompressed_size
 #include "gemmi/smcif.hpp"         // for make_small_structure_from_block
 #include "gemmi/small.hpp"         // for SmallStructure
 #include "gemmi/interop.hpp"       // for atom_to_site, mx_to_sx_structure
-#include "gemmi/chemcomp_xyz.hpp"
-#include "gemmi/remarks.hpp"
-
-#define GEMMI_READ_CIF_IMPLEMENTATION
-#include "gemmi/read_cif.hpp" // for read_cif_gz, read_mmjson_gz
-#define GEMMI_READ_COOR_IMPLEMENTATION
-#include "gemmi/read_coor.hpp"  // for read_structure_gz
+#include "gemmi/chemcomp_xyz.hpp"  // for make_structure_from_chemcomp_block
+#include "gemmi/read_cif.hpp"      // for read_cif_gz, read_mmjson_gz
+#include "gemmi/mmread_gz.hpp"     // for read_structure_gz
 
 #include "common.h"
 #include <pybind11/stl.h>
@@ -49,15 +47,14 @@ void add_cif_read(py::module& cif) {
 
 void add_read_structure(py::module& m) {
   m.def("read_structure", [](const std::string& path, bool merge,
-                             CoorFormat format) {
-          Structure* st = new Structure(read_structure_gz(path, format));
-          if (!st->raw_remarks.empty())
-            read_metadata_from_remarks(*st);
+                             CoorFormat format, cif::Document* save_doc) {
+          Structure* st = new Structure(read_structure_gz(path, format, save_doc));
           if (merge)
             st->merge_chain_parts();
           return st;
         }, py::arg("path"), py::arg("merge_chain_parts")=true,
            py::arg("format")=CoorFormat::Unknown,
+           py::arg("save_doc")=nullptr,
         "Reads a coordinate file into Structure.");
   m.def("make_structure_from_block", &make_structure_from_block,
         py::arg("block"), "Takes mmCIF block and returns Structure.");
@@ -66,9 +63,7 @@ void add_read_structure(py::module& m) {
           PdbReadOptions options;
           options.max_line_length = max_line_length;
           options.split_chain_on_ter = split_chain_on_ter;
-          Structure* st = new Structure(read_pdb_string(s, "string", options));
-          read_metadata_from_remarks(*st);
-          return st;
+          return new Structure(read_pdb_string(s, "string", options));
         }, py::arg("s"), py::arg("max_line_length")=0,
            py::arg("split_chain_on_ter")=false, "Reads a string as PDB file.");
   m.def("read_pdb", [](const std::string& path, int max_line_length,
@@ -76,9 +71,7 @@ void add_read_structure(py::module& m) {
           PdbReadOptions options;
           options.max_line_length = max_line_length;
           options.split_chain_on_ter = split_chain_on_ter;
-          Structure* st = new Structure(read_pdb_gz(path, options));
-          read_metadata_from_remarks(*st);
-          return st;
+          return new Structure(read_pdb_gz(path, options));
         }, py::arg("filename"), py::arg("max_line_length")=0,
            py::arg("split_chain_on_ter")=false);
 
