@@ -21,7 +21,7 @@ void add_scaling(py::module& m) {
     .def_readwrite("k_sol", &Scaling::k_sol)
     .def_readwrite("b_sol", &Scaling::b_sol)
     .def("prepare_points", &Scaling::prepare_points,
-         py::arg("calc"), py::arg("obs"), py::arg("mask")=FPhiData())
+         py::arg("calc"), py::arg("obs"), py::arg("mask")=static_cast<FPhiData*>(nullptr))
     .def("fit_isotropic_b_approximately", &Scaling::fit_isotropic_b_approximately)
     .def("fit_parameters", &Scaling::fit_parameters)
     .def("get_overall_scale_factor", &Scaling::get_overall_scale_factor, py::arg("hkl"))
@@ -38,7 +38,7 @@ void add_scaling(py::module& m) {
     })
     .def("get_solvent_scale", py::vectorize(&Scaling::get_solvent_scale), py::arg("stol2"))
     .def("scale_data", &Scaling::scale_data,
-         py::arg("asu_data"), py::arg("mask_data")=FPhiData())
+         py::arg("asu_data"), py::arg("mask_data")=static_cast<FPhiData*>(nullptr))
     .def("scale_value", &Scaling::scale_value,
          py::arg("hkl"), py::arg("f_value"), py::arg("mask_value"))
     ;
