@@ -33,7 +33,15 @@ void setup_for_crd(Structure& st) {
       for (Residue& res : chain.residues) {
         size_t n = chain.name.size();
         assert(res.subchain[n] == 'x');
-        res.subchain[n] = '_';
+        if (n < 3)
+          // Translate Axp to A_p. Refmac reads a single chain id from crd,
+          // but it recognizes '_' as separator and sets auth_asym_id to A.
+          res.subchain[n] = '_';
+        else
+          // Refmac checks for '_' only in 2nd and 3rd place. If the chain id
+          // length is 3-4 (it's rare), put the bare id, without appending _x.
+          // It will result in incorrect _struct_asym, but Refmac should work.
+          res.subchain = chain.name;
       }
   ensure_entities(st);
   deduplicate_entities(st);
@@ -367,6 +375,14 @@ static void add_restraint_row(cif::Loop& restr_loop,
   }
 }
 
+// For printing _restr.val_obs for torsion angles [-180,180].
+// Numerical errors still can change the output, but it's less likely.
+static double make_torsion_angle_consistent(double d) {
+  if (d < -179.99999)  // ensure 180.000 instead of -180.000,
+    d += 360.;
+  return std::fabs(d) > 1e-5 ? d : 0.;  // ensure 0.000 instead of -0.000
+}
+
 static void add_restraints(const Topo::Rule rule, const Topo& topo,
                            cif::Loop& restr_loop, int (&counters)[6],
                            const UnitCell* cell=nullptr) {
@@ -401,7 +417,7 @@ static void add_restraints(const Topo::Rule rule, const Topo& topo,
                       t.restr->label, std::to_string(t.restr->period),
                       {t.atoms[0], t.atoms[1], t.atoms[2], t.atoms[3]},
                       t.restr->value, t.restr->esd, NAN, NAN,
-                      deg(t.calculate()));
+                      make_torsion_angle_consistent(deg(t.calculate())));
   } else if (rule.rkind == Topo::RKind::Chirality) {
     const Topo::Chirality& t = topo.chirs[rule.index];
     add_restraint_row(restr_loop, "CHIR", ++counters[3],
@@ -413,11 +429,13 @@ static void add_restraints(const Topo::Rule rule, const Topo& topo,
     const Topo::Plane& t = topo.planes[rule.index];
     ++counters[4];
     auto coeff = find_best_plane(t.atoms);
-    for (const Atom* atom : t.atoms)
+    for (const Atom* atom : t.atoms) {
+      double calc = get_distance_from_plane(atom->pos, coeff);
       add_restraint_row(restr_loop, "PLAN", counters[4], t.restr->label, ".",
                         {atom},
                         t.restr->esd, NAN, NAN, NAN,
-                        get_distance_from_plane(atom->pos, coeff));
+                        std::fabs(calc) > 1e-5 ? calc : 0.);
+    }
   }
 }
 
@@ -505,16 +523,17 @@ cif::Block prepare_rst(const Topo& topo, const MonLib& monlib, const UnitCell& c
   return block;
 }
 
-cif::Document prepare_refmac_crd(const Structure& st, const Topo& topo,
+cif::Document prepare_refmac_crd(Structure& st, const Topo& topo,
                                  const MonLib& monlib, HydrogenChange h_change) {
   cif::Document doc;
   std::string info_comment = "# Refmac CRD file generated with gemmi " GEMMI_VERSION
                              "\n# Monomer library version: " + monlib.lib_version;
+  std::vector<std::string> resnames = st.models.at(0).get_all_residue_names();
+  shorten_ccd_codes(st);
   doc.blocks.push_back(prepare_crd(st, topo, h_change, info_comment));
   doc.blocks.push_back(prepare_rst(topo, monlib, st.cell));
 
   doc.blocks.emplace_back("for_refmac_mmcif");
-  std::vector<std::string> resnames = st.models.at(0).get_all_residue_names();
   for (const std::string& resname : resnames) {
     auto it = monlib.monomers.find(resname);
     if (it != monlib.monomers.end()) {
