@@ -1,6 +1,7 @@
 // Copyright 2017-2022 Global Phasing Ltd.
 
 #include <gemmi/polyheur.hpp>
+#include <gemmi/resinfo.hpp>   // for find_tabulated_residue
 
 namespace gemmi {
 
@@ -62,6 +63,20 @@ PolymerType check_polymer_type(const ConstResidueSpan& span, bool ignore_entity_
   return PolymerType::Unknown;
 }
 
+std::string make_one_letter_sequence(const ConstResidueSpan& polymer) {
+  std::string seq;
+  const Residue* prev = nullptr;
+  PolymerType ptype = check_polymer_type(polymer);
+  for (const Residue& residue : polymer.first_conformer()) {
+    ResidueInfo info = find_tabulated_residue(residue.name);
+    if (prev && !are_connected2(*prev, residue, ptype))
+      seq += '-';
+    seq += (info.one_letter_code != ' ' ? info.one_letter_code : 'X');
+    prev = &residue;
+  }
+  return seq;
+}
+
 static std::vector<Residue>::iterator infer_polymer_end(Chain& chain, PolymerType ptype) {
   auto b = chain.residues.begin();
   auto e = chain.residues.end();
@@ -147,11 +162,14 @@ void add_entity_ids(Structure& st, bool overwrite) {
   for (Model& model : st.models)
     for (Chain& chain : model.chains)
       for (ResidueSpan& sub : chain.subchains()) {
-        if (Entity* ent = st.get_entity_of(sub))
-          for (Residue& res : sub) {
+        if (Entity* ent = st.get_entity_of(sub)) {
+          for (Residue& res : sub)
             if (overwrite || res.entity_id.empty())
               res.entity_id = ent->name;
-          }
+        } else if (overwrite) {
+          for (Residue& res : sub)
+            res.entity_id.clear();
+        }
       }
 }
 
@@ -318,17 +336,21 @@ void change_ccd_code(Structure& st, const std::string& old, const std::string& n
     }
 }
 
+template <size_t I, typename T1, typename T2>
+bool in_vector_at(T1& x, std::vector<T2>& v) {
+  for (const auto& el : v)
+    if (std::get<I>(el) == x)
+      return true;
+  return false;
+}
+
 void shorten_ccd_codes(Structure& st) {
   // find all long residue names in both models and sequences
-  auto already_in = [&](const std::string& name) -> bool {
-    return in_vector_f([&](const OldToNew& x) { return x.old == name; },
-                       st.shortened_ccd_codes);
-  };
   for (Model& model : st.models)
     for (Chain& chain : model.chains)
       for (Residue& res : chain.residues)
-        if (res.name.size() > 3 && !already_in(res.name))
-          st.shortened_ccd_codes.push_back({res.name, {}});
+        if (res.name.size() > 3 && !in_vector_at<0>(res.name, st.shortened_ccd_codes))
+          st.shortened_ccd_codes.emplace_back(res.name, "");
   for (const Entity& ent : st.entities)
     for (const std::string& mon_ids : ent.full_sequence) {
       for (size_t start = 0;;) {
@@ -336,32 +358,33 @@ void shorten_ccd_codes(Structure& st) {
         size_t len = std::min(end, mon_ids.size()) - start;
         if (len > 3) {
           std::string s(mon_ids, start, len);
-          if (!already_in(s))
-            st.shortened_ccd_codes.push_back({s, {}});
+          if (!in_vector_at<0>(s, st.shortened_ccd_codes))
+            st.shortened_ccd_codes.emplace_back(s, "");
         }
         if (end == std::string::npos)
           break;
         start = end + 1;
       }
     }
+  // the first try on renaming: ABCDE -> ~DE
+  for (auto& old_new : st.shortened_ccd_codes) {
+    const std::string& old = old_new.first;
+    char short_code[4] = {'~', *(old.end()-2), *(old.end()-1), '\0'};
+    if (!in_vector_at<1>(short_code, st.shortened_ccd_codes))
+      old_new.second = short_code;
+  }
   // pick a new residue name and call change_ccd_code()
-  for (OldToNew& item : st.shortened_ccd_codes) {
-    std::string short_code = item.old.substr(0, 3);
-    short_code[2] = '~';
-    // if short_code it's already used, try X[0-9]~ and then [0-9][0-9]~
-    char c0 = '0', c1 = '0';
-    while (in_vector_f([&](const OldToNew& x) { return x.new_ == short_code; },
-                       st.shortened_ccd_codes)) {
-      short_code[1] = c1++;
-      if (c1 > '9') {
-        short_code[0] = c0++;
-        c1 = '0';
-        if (c0 > 'Z')  // shouldn't happen
-          break;
-      }
+  int i = -1;
+  for (auto& old_new : st.shortened_ccd_codes) {
+    // If ~DE was not unique, use ~00, ~01, ...
+    // After ~99, the middle character will be punctuation or letter.
+    // After ~Z9 (430+ names), we give up and the names will be empty.
+    while (old_new.second.empty() && ++i < 'Z'*10) {
+      char short_code[4] = {'~', char('0' + i/10), char('0' + i%10), '\0'};
+      if (!in_vector_at<1>(short_code, st.shortened_ccd_codes))
+        old_new.second = short_code;
     }
-    item.new_ = short_code;
-    change_ccd_code(st, item.old, item.new_);
+    change_ccd_code(st, old_new.first, old_new.second);
   }
 }
 
