@@ -230,7 +230,7 @@ void place_hydrogens(const Topo& topo, const Atom& atom,
       h.pos = arbitrary_position_from_angle(heavy.pos, atom.pos, h.dist, theta);
     if (std::isnan(h.pos.x)) {
       h.pos = Position(0, 0, 0);
-      giveup("bonded atoms are exactly overlapping.");
+      giveup("bonded atoms are exactly overlapping (case 1).");
     }
     if (hs.size() == 2) {
       // I think we can assume the two hydrogens are symmetric.
@@ -322,7 +322,7 @@ void place_hydrogens(const Topo& topo, const Atom& atom,
     Vec3 u20 = (known[1].pos - atom.pos).normalized();
     Vec3 v = u10.cross(u20);
     if (std::isnan(v.x))
-      giveup("bonded atoms are exactly overlapping.");
+      giveup("bonded atoms are exactly overlapping (case 2).");
     Vec3 d = a * u10 + b * u20;
     double dist_sin = hs[0].dist * std::sin(hh_half);
     double dist_cos = hs[0].dist * std::cos(hh_half);
@@ -357,10 +357,11 @@ void place_hydrogens(const Topo& topo, const Atom& atom,
     };
     SMat33<double> m{1., 1., 1., u10.dot(u20), u10.dot(u30), u20.dot(u30)};
     Vec3 rhs(cos_tetrahedral(0), cos_tetrahedral(1), cos_tetrahedral(2));
-    Vec3 abc = m.inverse().multiply(rhs);
+    double det = m.determinant();
+    if (std::fabs(det) < 1e-12)
+      giveup("tetrahedral configuration with four co-planar atoms.");
+    Vec3 abc = m.inverse_(det).multiply(rhs);
     Vec3 h_dir = abc.x * u10 + abc.y * u20 + abc.z * u30;
-    if (std::isnan(h_dir.x))
-      giveup("bonded atoms are exactly overlapping.");
     hs[0].pos = atom.pos + Position(h_dir.changed_magnitude(hs[0].dist));
   }
 }
