@@ -26,7 +26,7 @@ std::unique_ptr<ChemComp> make_chemcomp_with_restraints(const Residue& res) {
     if (el == El::D)
       el = El::H;
     const std::string& chem_type = el.uname();
-    cc->atoms.push_back(ChemComp::Atom{a.name, el, float(a.charge), chem_type, a.pos});
+    cc->atoms.push_back(ChemComp::Atom{a.name, "", el, float(a.charge), chem_type, a.pos});
   }
   // prepare pairs of atoms
   struct Pair {
@@ -169,12 +169,22 @@ static const ChemLink* setup_link_if_matches(Topo::Link& link, const MonLib& mon
     if (invert) {
       std::swap(link.res1, link.res2);
       std::swap(link.alt1, link.alt2);
+      std::swap(link.atom1_name_id, link.atom2_name_id);
       std::swap(link.aliasing1, link.aliasing2);
     }
   }
   return match;
 }
 
+static int atom_name_id(const std::string& s) {
+  int id = 0;
+  if (s.size() > 0)  // there is '\0' at the end
+    id = (s[0] << 24 | s[1] << 16);
+  if (s.size() > 2)
+    id |= (s[2] << 8 | s[3]);
+  return id | 0x20202020;
+}
+
 static void add_polymer_links(PolymerType polymer_type,
                               const Topo::ResInfo& ri1,
                               Topo::ResInfo& ri2,
@@ -230,6 +240,8 @@ static void add_polymer_links(PolymerType polymer_type,
             // and CA atoms in 2+ conformations making both CIS and TRANS links.
             link.alt1 = a1.altloc;
             link.alt2 = a2.altloc;
+            link.atom1_name_id = atom_name_id(a1.name);
+            link.atom2_name_id = atom_name_id(a2.name);
             if (groups_ok) {
               // Deciding CIS/TRANS based on omega angle.
               char alt = a1.altloc_or(a2.altloc_or('*'));
@@ -286,6 +298,8 @@ static void add_polymer_links(PolymerType polymer_type,
               in_nucleotide_bond_distance(&a1, &a2)) {
             link.alt1 = a1.altloc;
             link.alt2 = a2.altloc;
+            link.atom1_name_id = atom_name_id(a1.name);
+            link.atom2_name_id = atom_name_id(a2.name);
             if (groups_ok) {
               link.link_id = "p";
             } else if (monlib) {
@@ -320,9 +334,8 @@ double Topo::ideal_chiral_abs_volume(const Chirality &ch) const {
 }
 
 std::vector<Topo::Rule> Topo::apply_restraints(const Restraints& rt,
-                                               Residue& res, Residue* res2,
-                                               char altloc1, char altloc2,
-                                               bool require_alt) {
+                                               Residue& res, Residue* res2, Asu asu,
+                                               char altloc1, char altloc2, bool require_alt) {
   std::string altlocs;
   if (altloc1 == '\0' && altloc2 == '\0') {
     add_distinct_altlocs(res, altlocs);
@@ -340,7 +353,7 @@ std::vector<Topo::Rule> Topo::apply_restraints(const Restraints& rt,
           bool with_alt = at1->altloc || at2->altloc;
           if (with_alt || !require_alt) {
             rules.push_back({RKind::Bond, bonds.size()});
-            bonds.push_back({&bond, {{at1, at2}}});
+            bonds.push_back({&bond, {{at1, at2}}, asu});
           }
           if (!with_alt)
             break;
@@ -433,7 +446,8 @@ void Topo::apply_restraints_from_link(Link& link, const MonLib& monlib) {
     rt = rt_copy.get();
     rt_storage.push_back(std::move(rt_copy));
   }
-  link.link_rules = apply_restraints(*rt, *link.res1, link.res2, link.alt1, link.alt2, false);
+  link.link_rules = apply_restraints(*rt, *link.res1, link.res2, link.asu,
+                                     link.alt1, link.alt2, false);
 }
 
 // see comments above the declaration
@@ -544,8 +558,8 @@ void Topo::initialize_refmac_topology(Structure& st, Model& model0,
                   try {
                     chem_mod->apply_to(*cc_copy, mod.alias);
                   } catch(std::runtime_error& e) {
-                    err("failed to apply modification " + chem_mod->id
-                        + " to " + ri.res->name + ": " + e.what());
+                    err(cat("failed to apply modification ", chem_mod->id,
+                            " to ", ri.res->name, ": ", e.what()));
                   }
                 } else {
                   err("modification not found: " + mod.id);
@@ -586,10 +600,10 @@ void Topo::apply_all_restraints(const MonLib& monlib) {
         apply_restraints_from_link(link, monlib);
       // monomer restraints
       auto it = ri.chemcomps.cbegin();
-      ri.monomer_rules = apply_restraints(it->cc->rt, *ri.res, nullptr,
+      ri.monomer_rules = apply_restraints(it->cc->rt, *ri.res, nullptr, Asu::Same,
                                           it->altloc, '\0', /*require_alt=*/false);
       while (++it != ri.chemcomps.end()) {
-        auto rules = apply_restraints(it->cc->rt, *ri.res, nullptr,
+        auto rules = apply_restraints(it->cc->rt, *ri.res, nullptr, Asu::Same,
                                       it->altloc, '\0', /*require_alt=*/true);
         vector_move_extend(ri.monomer_rules, std::move(rules));
       }
@@ -620,13 +634,35 @@ void Topo::create_indices() {
       plane_index.emplace(atom, &plane);
 }
 
+Topo::Link* Topo::find_polymer_link(const AtomAddress& aa1, const AtomAddress& aa2) {
+  auto are_bonded = [&](const Link& link, const AtomAddress& a1, const AtomAddress& a2) {
+    return a1.res_id.matches_noseg(*link.res1) &&
+           a2.res_id.matches_noseg(*link.res2) &&
+           a1.altloc == link.alt1 &&
+           a2.altloc == link.alt2 &&
+           atom_name_id(a1.atom_name) == link.atom1_name_id &&
+           atom_name_id(a2.atom_name) == link.atom2_name_id;
+  };
+  if (aa1.chain_name != aa2.chain_name)
+    return nullptr;
+  for (ChainInfo& ci : chain_infos)
+    if (aa1.chain_name == ci.chain_ref.name) {
+      for (ResInfo& ri : ci.res_infos)
+        for (Link& link : ri.prev) {
+          assert(link.res1 && link.res2);
+          if (are_bonded(link, aa1, aa2) || are_bonded(link, aa2, aa1))
+            return &link;
+        }
+    }
+  return nullptr;
+}
+
 // Tries to construct Topo::Link and append it to extras.
 // Side-effects: it may modify conn.link_id and add ChemLink to monlib.links.
 void Topo::setup_connection(Connection& conn, Model& model0, MonLib& monlib,
                             bool ignore_unknown_links) {
   if (conn.link_id == "gap") {
-    Link* polymer_link = find_polymer_link(conn.partner1, conn.partner2);
-    if (polymer_link)
+    if (Link* polymer_link = find_polymer_link(conn.partner1, conn.partner2))
       polymer_link->link_id.clear();  // disable polymer link
     return;
   }
@@ -674,11 +710,14 @@ void Topo::setup_connection(Connection& conn, Model& model0, MonLib& monlib,
   // or if it matches residue-specific link from monomer library, use it;
   // otherwise, LINK is repetition of TRANS/CIS, so ignore LINK.
   if (Link* polymer_link = find_polymer_link(conn.partner1, conn.partner2)) {
-    if (conn.link_id.empty() && !cif::is_null(polymer_link->link_id) &&
-        polymer_link->link_id != "gap" &&
-        (!match || (match->side1.comp.empty() && match->side2.comp.empty())))
+    if (!conn.link_id.empty() ||  // conn has explicit link name
+        // polymer link was not really identified
+        cif::is_null(polymer_link->link_id) || polymer_link->link_id == "gap" ||
+        // conn is residue-specific
+        (match && !(match->side1.comp.empty() && match->side2.comp.empty())))
+      polymer_link->link_id.clear();  // disable polymer link, use conn
+    else
       return;
-    polymer_link->link_id.clear();  // disable polymer link
   }
 
   if (match) {
@@ -800,7 +839,7 @@ using NeighMap = std::unordered_multimap<int, Neigh>;
 
 // Assumes no hydrogens in the residue.
 // Position and serial number are not assigned for new atoms.
-void add_hydrogens_without_positions(Topo::ResInfo& ri, const NeighMap& neighbors) {
+void add_hydrogens_without_positions(Topo::ResInfo& ri, const NeighMap& neighbor_altlocs) {
   std::vector<Atom>& atoms = ri.res->atoms;
   // Add H atom for each conformation (altloc) of the parent atom and its
   // first neighbors.
@@ -811,15 +850,22 @@ void add_hydrogens_without_positions(Topo::ResInfo& ri, const NeighMap& neighbor
     float parent_occ = atoms[i].occ;
     std::map<char, float> altlocs; // altloc + occupancy
     if (parent_alt == '\0') {
-      float max_occ = 1.001f;
-      auto range = neighbors.equal_range(atoms[i].serial);
+      // We could have neighbor 1 with altlocs A,B and neighbor 2 with C,D.
+      // Or A,B and A,B,C. Or, neighbor 1 could have different occupancy
+      // of altloc A than neighbor 2. We don't want to check for all possible
+      // weird cases. Only ensure that the added H has total occupancy <= 1.
+      float occ_sum = 0;
+      auto range = neighbor_altlocs.equal_range(atoms[i].serial);
       for (auto it = range.first; it != range.second; ++it) {
         const Neigh& neigh = it->second;
-        if (neigh.alt && altlocs.count(neigh.alt) == 0 && neigh.occ < max_occ) {
-          altlocs.emplace(neigh.alt, neigh.occ * parent_occ);
-          max_occ -= neigh.occ;
+        if (altlocs.count(neigh.alt) == 0 && occ_sum + neigh.occ <= 1.001f) {
+          occ_sum += neigh.occ;
+          altlocs.emplace(neigh.alt, neigh.occ);
         }
       }
+      if (occ_sum - parent_occ > 0.001f)
+        for (auto& it : altlocs)
+          it.second *= parent_occ;
     }
     if (altlocs.empty())
       altlocs.emplace(parent_alt, parent_occ);
@@ -851,9 +897,9 @@ void add_hydrogens_without_positions(Topo::ResInfo& ri, const NeighMap& neighbor
   }
 }
 
-NeighMap prepare_neighbor_data(Topo& topo, const MonLib& monlib) {
+NeighMap prepare_neighbor_altlocs(Topo& topo, const MonLib& monlib) {
   // disable warnings here, so they are not printed twice
-  std::streambuf *warnings_orig = nullptr;
+  std::streambuf* warnings_orig = nullptr;
   if (topo.warnings)
     warnings_orig = topo.warnings->rdbuf(nullptr);
   // Prepare bonds. Fills topo.bonds, monomer_rules/link_rules and rt_storage,
@@ -868,8 +914,9 @@ NeighMap prepare_neighbor_data(Topo& topo, const MonLib& monlib) {
   for (const Topo::Bond& bond : topo.bonds) {
     const Atom* a1 = bond.atoms[0];
     const Atom* a2 = bond.atoms[1];
-    neighbors.emplace(a1->serial, Neigh{a2->altloc, a2->occ});
-    if (a1 != a2)
+    if (a2->altloc)
+      neighbors.emplace(a1->serial, Neigh{a2->altloc, a2->occ});
+    if (a1->altloc && a1 != a2)
       neighbors.emplace(a2->serial, Neigh{a1->altloc, a1->occ});
   }
   return neighbors;
@@ -921,8 +968,13 @@ prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
         if (!cc.has_atom(atom.name)) {
           std::string msg = "definition not found for "
                           + atom_str(chain_info.chain_ref, *ri.res, atom);
-          if (ri.orig_chemcomp && ri.orig_chemcomp->has_atom(atom.name))
+          if (ri.orig_chemcomp && ri.orig_chemcomp->has_atom(atom.name)) {
             msg += " (linkage should remove this atom)";
+          } else {
+            auto it = cc.find_atom_by_old_name(atom.name);
+            if (it != cc.atoms.end())
+              cat_to(msg, " (replace ", atom.name, " with ", it->id, ')');
+          }
           topo->err(msg);
         }
       }
@@ -932,13 +984,13 @@ prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
   if (h_change == HydrogenChange::ReAdd ||
       h_change == HydrogenChange::ReAddButWater ||
       h_change == HydrogenChange::ReAddKnown) {
-    NeighMap neighbors = prepare_neighbor_data(*topo, monlib);
+    NeighMap neighbor_altlocs = prepare_neighbor_altlocs(*topo, monlib);
     for (Topo::ChainInfo& chain_info : topo->chain_infos)
       for (Topo::ResInfo& ri : chain_info.res_infos) {
         Residue& res = *ri.res;
         if (ri.orig_chemcomp != nullptr &&
             (h_change == HydrogenChange::ReAdd || !res.is_water())) {
-          add_hydrogens_without_positions(ri, neighbors);
+          add_hydrogens_without_positions(ri, neighbor_altlocs);
 
           // a special handling of HIS for compatibility with Refmac
           if (res.name == "HIS") {
@@ -1022,4 +1074,19 @@ prepare_topology(Structure& st, MonLib& monlib, size_t model_index,
   return topo;
 }
 
+std::vector<AtomAddress> find_missing_atoms(const Topo& topo, bool including_hydrogen) {
+  std::vector<AtomAddress> ret;
+  for (const Topo::ChainInfo& chain_info : topo.chain_infos)
+    for (const Topo::ResInfo& ri : chain_info.res_infos) {
+      // check only the first conformation
+      const Topo::FinalChemComp& fcc = ri.chemcomps.at(0);
+      char altloc = fcc.altloc != '\0' ? fcc.altloc : '*';
+      for (const ChemComp::Atom& at : fcc.cc->atoms)
+        if ((including_hydrogen || !at.is_hydrogen()) &&
+            ri.res->find_atom(at.id, altloc) == nullptr)
+          ret.emplace_back(chain_info.chain_ref.name, *ri.res, at.id, fcc.altloc);
+    }
+  return ret;
+}
+
 }
