@@ -206,7 +206,7 @@ class TestMol(unittest.TestCase):
               '_cell.', '_chem_comp.',
               '_diffrn.', '_diffrn_detector.', '_diffrn_radiation.',
               '_diffrn_source.',
-              '_entity.', '_entity_poly_seq.', '_entry.',
+              '_entity.', '_entity_poly.', '_entity_poly_seq.', '_entry.',
               '_exptl.', '_exptl_crystal.', '_pdbx_database_status.',
               '_pdbx_struct_assembly.', '_pdbx_struct_assembly_gen.',
               '_pdbx_struct_mod_residue.',
@@ -746,7 +746,7 @@ class TestMol(unittest.TestCase):
                           'ALA', 'NCY', 'MVA'])
         self.assertEqual(len(polymer), 10)
         self.assertEqual(polymer.length(), 8)
-        # The bond between 4 MVA(v) and 5 DSN(s) is betwenn C and OG
+        # The bond between 4 MVA(v) and 5 DSN(s) is between C and OG
         # (so it's not a peptide bond as expected). Depending on the heuristic
         # used to determine gaps, this sequence could have a gap in the middle.
         self.assertEqual(polymer.make_one_letter_sequence(), 'sAXvsAXv')
